package EV3;

import lejos.hardware.Button; //Zugriff auf die Button Klasse nun moeglich
import lejos.hardware.motor.EV3LargeRegulatedMotor; //Klasse fuer einen grossen EV3-Motor
import lejos.hardware.port.MotorPort; //Zugriff auf die Motorports moeglich (Interface) 
import lejos.hardware.port.SensorPort; //Zugriff auf die Sensorports moeglich (Interface) 
import lejos.hardware.sensor.EV3ColorSensor; //Zugriff auf die Klasse des Farbsensors

/**
 * @author Patric Renner
 * @version 2.0
 * @date 05.05.2018
 * 
 * Klasse fuer einen Regulated-Motor fuer Right Line Follow
 * 
 * Ein Line Follower Code der fuer alle Kurven geeignet ist.
 * 
 * Der Roboter auf der Linie zwischen schwarz und weiss entlang fahren zu lassen.
 * 
 * ACHTUNG: Roboter auf der rechten Seite der Linie aufstellen!
 * 
*/
public class RegulatedRightLineFollow {

	//Zwei neue Motor Objekte erzeugen:
	//motorA greift auf Port A zu
	//motorB greift auf Port D zu
	private EV3LargeRegulatedMotor motorA = new EV3LargeRegulatedMotor(MotorPort.A);
	private EV3LargeRegulatedMotor motorD = new EV3LargeRegulatedMotor(MotorPort.D);
	
	//Neues ColorSensor Objekt erstellen das auf den Port 1 zugreift
	private EV3ColorSensor colorSensor = new EV3ColorSensor(SensorPort.S1);
	private float [] farbwerteRGB; 
	
	//Variablen fuer die Kalibrierung:
	float mitte, schwarz, weiss;
	float scanA, scanB, scanC, scanD, scanE;
	int speed;

	//Der Sensor wird im Konstruktuor in den RGB-Modus versetzt
	//Dadurch koennen Farbwerte im Kommabereich ausgelesen werden
	RegulatedRightLineFollow() {
		colorSensor.setCurrentMode("RGB");
		
		//Erzeugen eines Float Arrays zum Speichern der RGB-Werte
		farbwerteRGB = new float [colorSensor.sampleSize()];
	}
		
		
	/*
	 * Methode um auf die gescanten Werte zu reagieren
	 */	
	private void scanWerteUeberpruefen() { 
		
		//------Kalibrierungseinstellungen: Werte die veraendert werden koennen------
		//Der Bereich in dem sich der Roboter bewegt:
		weiss = 0.65f;
		mitte = 0.35f;
		schwarz = 0.05f;
		
		//Reaktionsbereiche:
		//Je hoeher die Zahl, desto groesser ist der Unterschied in der Geschwindigkeit
		scanA = 0.0451f;
		scanB = 0.0951f;
		scanC = 0.1451f;
		scanD = 0.1951f;
		scanE = 0.2451f;
		
		//Hoechstgeschwindigkeit:
		speed = 150;
		//---------------------Kalibrierungseinstellungen Ende!-------------------
		
		
		//Die Power der Motoren einstellen zum Losfahren:
	    motorA.setSpeed(speed);
	    //Mit der zweiten Methode wird dem Motor 
	    //gesagt ob er nach vorne oder nach hinten faehrt
	    motorA.forward(); //Vor 
	    //motorA.backward(); //Zurueck

	    motorD.setSpeed(speed);
	    motorD.forward();
	
	     //Solange der Zurueck-Button nicht gedrueckt ist, wird das Programm ausgefuehrt
		 while (Button.ESCAPE.isUp()) {
			 
			//holt einen Farbwert und speichert ihn
			colorSensor.fetchSample(farbwerteRGB, 0); 
			//ab Wert 0 im Array ab
			
			//Bilden einer Summe der RGB-Werte
		    float summeFarbwerteRGB = farbwerteRGB[0] + farbwerteRGB[1] + farbwerteRGB[2];

		    /* Sobald der Wert hoeher wird befindet man sich eher im weissen Bereich
		    * -> also nach links steuern 
		    * 
		    * Sobald der Wert niedriger wird, befindet man sich eher im schwarzen Bereich
		    * -> also nach rechts steuern
		    *  
		    *  Um besser auf die Werte zu reagieren wurden die Intervalle von scanA-E festgelegt
			*  Weiter koennen die Geschwindigkeiten in den Methoden "zuWeitLinks" "zuWeitRechts" individuell angepasst werden
		    *  
		    **/
		    if (summeFarbwerteRGB > weiss) {  //Zu weit Rechts 
		    	zuWeitRechts(70, -70);

		    } else if (summeFarbwerteRGB > (weiss - scanA)  && summeFarbwerteRGB < weiss) {  //Zu weit Rechts
		    	zuWeitRechts(60, -50);

		    } else if (summeFarbwerteRGB > (weiss - scanB) && summeFarbwerteRGB < (weiss - scanA)) {  //Zu weit Rechts 
		    	zuWeitRechts(60, -20);

		    } else if (summeFarbwerteRGB > (weiss - scanC) && summeFarbwerteRGB < (weiss - scanB)) {  //Zu weit Rechts 
		    	zuWeitRechts(80, 5);

		    } else if (summeFarbwerteRGB > (weiss - scanD) && summeFarbwerteRGB < (weiss - scanC)) {  //Zu weit Rechts 
		    	zuWeitRechts(110, 60);

		    } else if (summeFarbwerteRGB > (weiss - scanE) && summeFarbwerteRGB < (weiss - scanD)) {  //Zu weit Rechts
		    	zuWeitRechts(130, 100);

		    } else if (summeFarbwerteRGB > mitte && summeFarbwerteRGB < (weiss - scanE)) { //Mitte
		    	zuWeitRechts(150, 130);

		    } else if (summeFarbwerteRGB > (schwarz + scanE) && summeFarbwerteRGB < mitte) { //Mitte
		    	zuWeitLinks(130, 150);

		    } else if (summeFarbwerteRGB > (schwarz + scanD) && summeFarbwerteRGB < (schwarz + scanE)) { //Zu weit Links 
		    	zuWeitLinks(100, 130);

		    } else if (summeFarbwerteRGB > (schwarz + scanC) && summeFarbwerteRGB < (schwarz + scanD)) { //Zu weit Links 
		    	zuWeitLinks(60, 110);

		    } else if (summeFarbwerteRGB > (schwarz + scanB) && summeFarbwerteRGB < (schwarz + scanC)) { //Zu weit Links 
		    	zuWeitLinks(5, 80);

		    } else if (summeFarbwerteRGB > (schwarz + scanA) && summeFarbwerteRGB < (schwarz + scanB)) { //Zu weit Links 
		    	zuWeitLinks(-20, 60);
			    
			} else if (summeFarbwerteRGB > schwarz && summeFarbwerteRGB < (schwarz + scanA)) { //Zu weit Links 
				zuWeitLinks(-50, 60);

		    } else { //Zu weit Links
		    	zuWeitLinks(-70, 70);
		    } 
		 }
	}
	
	/*
	 * Methode im weissen Bereich, um zu ueberpruefen ob der Roboter die Motoren nach vorne oder nach hinten drehen laesst:
	 */	
	public void zuWeitRechts (int vorA, int zurueckD) {
		motorA.setSpeed(vorA);
		if (vorA > 0) {
			motorA.forward();
		} else {
			motorA.backward();
		}

	    motorD.setSpeed(zurueckD);
		if (zurueckD > 0) {
			motorD.forward();
		} else {
			motorD.backward();
		}
	}
	
	/*
	 * Methode im schwarzen Bereich, um zu ueberpruefen ob der Roboter die Motoren nach vorne oder nach hinten drehen laesst:
	 */	
	public void zuWeitLinks (int zurueckA, int vorD) {
		motorA.setSpeed(zurueckA);
		if (zurueckA > 0) {
			motorA.forward();
		} else {
			motorA.backward();
		}
		
	    motorD.setSpeed(vorD);
		if (vorD > 0) {
			motorD.forward();
		} else {
			motorD.backward();
		}
	}
	
	
	public static void main(String[] args) {
		//Erzeugen eines neuen Objektes das die Methode scanWerteUeberpruefen aufruft:
		RegulatedRightLineFollow drive = new RegulatedRightLineFollow();
		drive.scanWerteUeberpruefen();
	}    	
}